from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.service_visibility import ServiceVisibility
from ..types import UNSET, Unset

T = TypeVar("T", bound="Service")


@_attrs_define
class Service:
    """
    Attributes:
        name (str): Name of the service
        id (Union[Unset, str]): Id of the service
        team_id (Union[Unset, str]): Team id of the service.
        description (Union[Unset, str]): Description field of the service that is generally used to provide a detailed
            information about the service.
        visibility (Union[Unset, ServiceVisibility]):
        tags (Union[Unset, List[str]]):
    """

    name: str
    id: Union[Unset, str] = UNSET
    team_id: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    visibility: Union[Unset, ServiceVisibility] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        id = self.id

        team_id = self.team_id

        description = self.description

        visibility: Union[Unset, str] = UNSET
        if not isinstance(self.visibility, Unset):
            visibility = self.visibility.value

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if team_id is not UNSET:
            field_dict["teamId"] = team_id
        if description is not UNSET:
            field_dict["description"] = description
        if visibility is not UNSET:
            field_dict["visibility"] = visibility
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        id = d.pop("id", UNSET)

        team_id = d.pop("teamId", UNSET)

        description = d.pop("description", UNSET)

        _visibility = d.pop("visibility", UNSET)
        visibility: Union[Unset, ServiceVisibility]
        if isinstance(_visibility, Unset):
            visibility = UNSET
        else:
            visibility = ServiceVisibility(_visibility)

        tags = cast(List[str], d.pop("tags", UNSET))

        service = cls(
            name=name,
            id=id,
            team_id=team_id,
            description=description,
            visibility=visibility,
            tags=tags,
        )

        service.additional_properties = d
        return service

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class RecipientType(str, Enum):
    ALL = "all"
    ESCALATION = "escalation"
    GROUP = "group"
    NONE = "none"
    SCHEDULE = "schedule"
    TEAM = "team"
    USER = "user"

    def __str__(self) -> str:
        return str(self.value)



from textual.message import Message

from tygenie.widgets.checkboxes import Checkboxes


class Tags(Checkboxes):

    class RemoveTag(Message):
        """Posted when a tag has to be removed"""

        def __init__(self, value: str = ""):
            self.value = value
            super().__init__()

    async def on_checkbox_changed(self, event):
        if event.value is False:  # unchecked
            event.checkbox.disabled = True
            if self.post_message(self.RemoveTag(value=event.checkbox.name)):
                if event.checkbox.name in self.names:
                    self.names.remove(event.checkbox.name)
                    await self.recompose()



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.notification_action_type import NotificationActionType
from ..models.notify_time import NotifyTime
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_notification_rule_step_payload import CreateNotificationRuleStepPayload
    from ..models.filter_ import Filter
    from ..models.notification_repeat import NotificationRepeat
    from ..models.schedule_recipient import ScheduleRecipient
    from ..models.time_restriction_interval import TimeRestrictionInterval


T = TypeVar("T", bound="CreateNotificationRulePayload")


@_attrs_define
class CreateNotificationRulePayload:
    """
    Attributes:
        name (str): Name of the notification rule
        action_type (NotificationActionType): Type of the action that notification rule will have
        enabled (bool): Defines if notification rule will be enabled or not when it is created
        criteria (Union[Unset, Filter]): Defines the conditions that will be checked before applying rules and type of
            the operations that will be applied on conditions
        notification_time (Union[Unset, List[NotifyTime]]): List of Time Periods that notification for schedule
            start/end will be sent
        time_restriction (Union[Unset, TimeRestrictionInterval]):
        schedules (Union[Unset, List['ScheduleRecipient']]): List of schedules that notification rule will be applied
            when on call of that schedule starts/ends. This field is valid for Schedule Start/End rules
        order (Union[Unset, int]): The order of the notification rule within the notification rules with the same action
            type
        steps (Union[Unset, List['CreateNotificationRuleStepPayload']]): List of steps that will be added to
            notification rule
        repeat (Union[Unset, NotificationRepeat]): The amount of time in minutes that notification steps will be
            repeatedly apply
    """

    name: str
    action_type: NotificationActionType
    enabled: bool
    criteria: Union[Unset, "Filter"] = UNSET
    notification_time: Union[Unset, List[NotifyTime]] = UNSET
    time_restriction: Union[Unset, "TimeRestrictionInterval"] = UNSET
    schedules: Union[Unset, List["ScheduleRecipient"]] = UNSET
    order: Union[Unset, int] = UNSET
    steps: Union[Unset, List["CreateNotificationRuleStepPayload"]] = UNSET
    repeat: Union[Unset, "NotificationRepeat"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        action_type = self.action_type.value

        enabled = self.enabled

        criteria: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.criteria, Unset):
            criteria = self.criteria.to_dict()

        notification_time: Union[Unset, List[str]] = UNSET
        if not isinstance(self.notification_time, Unset):
            notification_time = []
            for notification_time_item_data in self.notification_time:
                notification_time_item = notification_time_item_data.value
                notification_time.append(notification_time_item)

        time_restriction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.time_restriction, Unset):
            time_restriction = self.time_restriction.to_dict()

        schedules: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.schedules, Unset):
            schedules = []
            for schedules_item_data in self.schedules:
                schedules_item = schedules_item_data.to_dict()
                schedules.append(schedules_item)

        order = self.order

        steps: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.steps, Unset):
            steps = []
            for steps_item_data in self.steps:
                steps_item = steps_item_data.to_dict()
                steps.append(steps_item)

        repeat: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.repeat, Unset):
            repeat = self.repeat.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "actionType": action_type,
                "enabled": enabled,
            }
        )
        if criteria is not UNSET:
            field_dict["criteria"] = criteria
        if notification_time is not UNSET:
            field_dict["notificationTime"] = notification_time
        if time_restriction is not UNSET:
            field_dict["timeRestriction"] = time_restriction
        if schedules is not UNSET:
            field_dict["schedules"] = schedules
        if order is not UNSET:
            field_dict["order"] = order
        if steps is not UNSET:
            field_dict["steps"] = steps
        if repeat is not UNSET:
            field_dict["repeat"] = repeat

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_notification_rule_step_payload import CreateNotificationRuleStepPayload
        from ..models.filter_ import Filter
        from ..models.notification_repeat import NotificationRepeat
        from ..models.schedule_recipient import ScheduleRecipient
        from ..models.time_restriction_interval import TimeRestrictionInterval

        d = src_dict.copy()
        name = d.pop("name")

        action_type = NotificationActionType(d.pop("actionType"))

        enabled = d.pop("enabled")

        _criteria = d.pop("criteria", UNSET)
        criteria: Union[Unset, Filter]
        if isinstance(_criteria, Unset):
            criteria = UNSET
        else:
            criteria = Filter.from_dict(_criteria)

        notification_time = []
        _notification_time = d.pop("notificationTime", UNSET)
        for notification_time_item_data in _notification_time or []:
            notification_time_item = NotifyTime(notification_time_item_data)

            notification_time.append(notification_time_item)

        _time_restriction = d.pop("timeRestriction", UNSET)
        time_restriction: Union[Unset, TimeRestrictionInterval]
        if isinstance(_time_restriction, Unset):
            time_restriction = UNSET
        else:
            time_restriction = TimeRestrictionInterval.from_dict(_time_restriction)

        schedules = []
        _schedules = d.pop("schedules", UNSET)
        for schedules_item_data in _schedules or []:
            schedules_item = ScheduleRecipient.from_dict(schedules_item_data)

            schedules.append(schedules_item)

        order = d.pop("order", UNSET)

        steps = []
        _steps = d.pop("steps", UNSET)
        for steps_item_data in _steps or []:
            steps_item = CreateNotificationRuleStepPayload.from_dict(steps_item_data)

            steps.append(steps_item)

        _repeat = d.pop("repeat", UNSET)
        repeat: Union[Unset, NotificationRepeat]
        if isinstance(_repeat, Unset):
            repeat = UNSET
        else:
            repeat = NotificationRepeat.from_dict(_repeat)

        create_notification_rule_payload = cls(
            name=name,
            action_type=action_type,
            enabled=enabled,
            criteria=criteria,
            notification_time=notification_time,
            time_restriction=time_restriction,
            schedules=schedules,
            order=order,
            steps=steps,
            repeat=repeat,
        )

        create_notification_rule_payload.additional_properties = d
        return create_notification_rule_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from io import BytesIO
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, File, Unset

T = TypeVar("T", bound="AddAttachmentBody")


@_attrs_define
class AddAttachmentBody:
    """
    Attributes:
        file (File): Attachment file to be uploaded
        user (Union[Unset, str]): Display name of the request owner
        index_file (Union[Unset, str]): Name of html file which will be shown when attachment clicked on UI
    """

    file: File
    user: Union[Unset, str] = UNSET
    index_file: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        file = self.file.to_tuple()

        user = self.user

        index_file = self.index_file

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "file": file,
            }
        )
        if user is not UNSET:
            field_dict["user"] = user
        if index_file is not UNSET:
            field_dict["indexFile"] = index_file

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        file = self.file.to_tuple()

        user = self.user if isinstance(self.user, Unset) else (None, str(self.user).encode(), "text/plain")

        index_file = (
            self.index_file
            if isinstance(self.index_file, Unset)
            else (None, str(self.index_file).encode(), "text/plain")
        )

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = (None, str(prop).encode(), "text/plain")

        field_dict.update(
            {
                "file": file,
            }
        )
        if user is not UNSET:
            field_dict["user"] = user
        if index_file is not UNSET:
            field_dict["indexFile"] = index_file

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        file = File(payload=BytesIO(d.pop("file")))

        user = d.pop("user", UNSET)

        index_file = d.pop("indexFile", UNSET)

        add_attachment_body = cls(
            file=file,
            user=user,
            index_file=index_file,
        )

        add_attachment_body.additional_properties = d
        return add_attachment_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import re
import httpx
import semver
from textual import on, work
from textual.app import ComposeResult
from textual.containers import Horizontal
from textual.message import Message
from textual.reactive import reactive
from textual.widget import Widget
from textual.widgets import Button
import tygenie.logger as ty_logger

from tygenie import consts
from tygenie.config import ty_config
from tygenie.widgets.input import TagValueInput


class AlertActionContainer(Widget):

    tag_value: reactive = reactive("", recompose=True)
    version: reactive = reactive(f"v{consts.VERSION}", recompose=True)

    async def watch_tag_value(self):
        input = self.query_one("#tag_alert", Button)
        input.label = f"Tag alert '{self.tag_value}'"
        await self.recompose()

    async def watch_version(self):
        self.border_subtitle = self.version
        await self.recompose()

    def __init__(self, tag_value: str = "", **kwargs) -> None:
        super().__init__(**kwargs)
        if tag_value == "":
            tag_value = ty_config.tygenie.get("default_tag", "")

        self.set_reactive(AlertActionContainer.tag_value, tag_value)

    def compose(self) -> ComposeResult:
        self.border_title = "Actions"
        with Horizontal(id="alert_action_horizontal_container"):
            yield TagValueInput(id="tag_value_container")
            yield Button(
                label=f"Tag alert '{self.tag_value}'", name="tag_alert", id="tag_alert"
            )
            yield Button(
                label="Open in webbrowser",
                name="open_in_webbrowser",
                id="open_in_webbrowser",
            )
            yield Button(label="Add note", name="add_note", id="add_note")

    class OpenInBrowser(Message):
        """A message to indicate that we have to open selected alert in webbrowser"""

    class AddNote(Message):
        """A message to indicate that we have to open selected alert alertmanager link"""

    class AddTag(Message):
        """A message to indicate that we have to open selected alert alertmanager link"""

    class CheckTygenieNewVersion(Message):
        """A message"""

    async def on_mount(self):
        self.send_check_tygenie_version_message()
        # Check every 6 hours for new version
        self.set_interval(6 * 60 * 60, self.send_check_tygenie_version_message)

    @work(exclusive=True, exit_on_error=True, thread=True)
    def send_check_tygenie_version_message(self):
        self.post_message(self.CheckTygenieNewVersion())

    def on_button_pressed(self, event: Button.Pressed):
        if event.button.name == "open_in_webbrowser":
            self.post_message(self.OpenInBrowser())
        if event.button.name == "add_note":
            self.post_message(self.AddNote())
        if event.button.name == "tag_alert":
            self.post_message(self.AddTag())

    @on(TagValueInput.TagValueChange)
    def update_tag_value(self, message: TagValueInput.TagValueChange):
        self.tag_value = message.label

    @on(CheckTygenieNewVersion)
    @work(exclusive=True, exit_on_error=True, thread=False)
    async def check_tygenie_new_version(self):
        ty_logger.logger.log("Checking latest Tygenie release...")
        try:
            async with httpx.AsyncClient() as client:
                data = await client.get(
                    url="https://api.github.com/repos/ovh/tygenie/releases/latest"
                )
                latest_version = re.sub("^v", "", data.json()["name"])

            if semver.compare(consts.VERSION, latest_version) < 0:
                ty_logger.logger.log(f"New version found: {latest_version}")
                self.version = (
                    f"v{consts.VERSION} - [$text-error]v{latest_version} available![/]"
                )
        except Exception:
            pass



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.base_integration_action_type import BaseIntegrationActionType
from ..models.create_integration_action_priority import CreateIntegrationActionPriority
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_integration_action_extra_properties import CreateIntegrationActionExtraProperties
    from ..models.integration_action_filter import IntegrationActionFilter
    from ..models.recipient import Recipient


T = TypeVar("T", bound="CreateIntegrationAction")


@_attrs_define
class CreateIntegrationAction:
    """
    Attributes:
        name (str):
        filter_ (IntegrationActionFilter):
        type (BaseIntegrationActionType):
        order (Union[Unset, int]):
        user (Union[Unset, str]):
        note (Union[Unset, str]):
        alias (Union[Unset, str]):
        source (Union[Unset, str]):
        message (Union[Unset, str]):
        description (Union[Unset, str]):
        entity (Union[Unset, str]):
        priority (Union[Unset, CreateIntegrationActionPriority]):
        custom_priority (Union[Unset, str]):
        append_attachments (Union[Unset, bool]):
        alert_actions (Union[Unset, List[str]]):
        ignore_alert_actions_from_payload (Union[Unset, bool]):
        recipients (Union[Unset, List['Recipient']]):
        responders (Union[Unset, List['Recipient']]):
        ignore_recipients_from_payload (Union[Unset, bool]):
        ignore_teams_from_payload (Union[Unset, bool]):
        tags (Union[Unset, List[str]]):
        ignore_tags_from_payload (Union[Unset, bool]):
        extra_properties (Union[Unset, CreateIntegrationActionExtraProperties]):
        ignore_extra_properties_from_payload (Union[Unset, bool]):
        ignore_responders_from_payload (Union[Unset, bool]):
    """

    name: str
    filter_: "IntegrationActionFilter"
    type: BaseIntegrationActionType
    order: Union[Unset, int] = UNSET
    user: Union[Unset, str] = UNSET
    note: Union[Unset, str] = UNSET
    alias: Union[Unset, str] = UNSET
    source: Union[Unset, str] = UNSET
    message: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    entity: Union[Unset, str] = UNSET
    priority: Union[Unset, CreateIntegrationActionPriority] = UNSET
    custom_priority: Union[Unset, str] = UNSET
    append_attachments: Union[Unset, bool] = UNSET
    alert_actions: Union[Unset, List[str]] = UNSET
    ignore_alert_actions_from_payload: Union[Unset, bool] = UNSET
    recipients: Union[Unset, List["Recipient"]] = UNSET
    responders: Union[Unset, List["Recipient"]] = UNSET
    ignore_recipients_from_payload: Union[Unset, bool] = UNSET
    ignore_teams_from_payload: Union[Unset, bool] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    ignore_tags_from_payload: Union[Unset, bool] = UNSET
    extra_properties: Union[Unset, "CreateIntegrationActionExtraProperties"] = UNSET
    ignore_extra_properties_from_payload: Union[Unset, bool] = UNSET
    ignore_responders_from_payload: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        filter_ = self.filter_.to_dict()

        type = self.type.value

        order = self.order

        user = self.user

        note = self.note

        alias = self.alias

        source = self.source

        message = self.message

        description = self.description

        entity = self.entity

        priority: Union[Unset, str] = UNSET
        if not isinstance(self.priority, Unset):
            priority = self.priority.value

        custom_priority = self.custom_priority

        append_attachments = self.append_attachments

        alert_actions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.alert_actions, Unset):
            alert_actions = self.alert_actions

        ignore_alert_actions_from_payload = self.ignore_alert_actions_from_payload

        recipients: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.recipients, Unset):
            recipients = []
            for recipients_item_data in self.recipients:
                recipients_item = recipients_item_data.to_dict()
                recipients.append(recipients_item)

        responders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.responders, Unset):
            responders = []
            for responders_item_data in self.responders:
                responders_item = responders_item_data.to_dict()
                responders.append(responders_item)

        ignore_recipients_from_payload = self.ignore_recipients_from_payload

        ignore_teams_from_payload = self.ignore_teams_from_payload

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        ignore_tags_from_payload = self.ignore_tags_from_payload

        extra_properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.extra_properties, Unset):
            extra_properties = self.extra_properties.to_dict()

        ignore_extra_properties_from_payload = self.ignore_extra_properties_from_payload

        ignore_responders_from_payload = self.ignore_responders_from_payload

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "filter": filter_,
                "type": type,
            }
        )
        if order is not UNSET:
            field_dict["order"] = order
        if user is not UNSET:
            field_dict["user"] = user
        if note is not UNSET:
            field_dict["note"] = note
        if alias is not UNSET:
            field_dict["alias"] = alias
        if source is not UNSET:
            field_dict["source"] = source
        if message is not UNSET:
            field_dict["message"] = message
        if description is not UNSET:
            field_dict["description"] = description
        if entity is not UNSET:
            field_dict["entity"] = entity
        if priority is not UNSET:
            field_dict["priority"] = priority
        if custom_priority is not UNSET:
            field_dict["customPriority"] = custom_priority
        if append_attachments is not UNSET:
            field_dict["appendAttachments"] = append_attachments
        if alert_actions is not UNSET:
            field_dict["alertActions"] = alert_actions
        if ignore_alert_actions_from_payload is not UNSET:
            field_dict["ignoreAlertActionsFromPayload"] = ignore_alert_actions_from_payload
        if recipients is not UNSET:
            field_dict["recipients"] = recipients
        if responders is not UNSET:
            field_dict["responders"] = responders
        if ignore_recipients_from_payload is not UNSET:
            field_dict["ignoreRecipientsFromPayload"] = ignore_recipients_from_payload
        if ignore_teams_from_payload is not UNSET:
            field_dict["ignoreTeamsFromPayload"] = ignore_teams_from_payload
        if tags is not UNSET:
            field_dict["tags"] = tags
        if ignore_tags_from_payload is not UNSET:
            field_dict["ignoreTagsFromPayload"] = ignore_tags_from_payload
        if extra_properties is not UNSET:
            field_dict["extraProperties"] = extra_properties
        if ignore_extra_properties_from_payload is not UNSET:
            field_dict["ignoreExtraPropertiesFromPayload"] = ignore_extra_properties_from_payload
        if ignore_responders_from_payload is not UNSET:
            field_dict["ignoreRespondersFromPayload"] = ignore_responders_from_payload

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_integration_action_extra_properties import CreateIntegrationActionExtraProperties
        from ..models.integration_action_filter import IntegrationActionFilter
        from ..models.recipient import Recipient

        d = src_dict.copy()
        name = d.pop("name")

        filter_ = IntegrationActionFilter.from_dict(d.pop("filter"))

        type = BaseIntegrationActionType(d.pop("type"))

        order = d.pop("order", UNSET)

        user = d.pop("user", UNSET)

        note = d.pop("note", UNSET)

        alias = d.pop("alias", UNSET)

        source = d.pop("source", UNSET)

        message = d.pop("message", UNSET)

        description = d.pop("description", UNSET)

        entity = d.pop("entity", UNSET)

        _priority = d.pop("priority", UNSET)
        priority: Union[Unset, CreateIntegrationActionPriority]
        if isinstance(_priority, Unset):
            priority = UNSET
        else:
            priority = CreateIntegrationActionPriority(_priority)

        custom_priority = d.pop("customPriority", UNSET)

        append_attachments = d.pop("appendAttachments", UNSET)

        alert_actions = cast(List[str], d.pop("alertActions", UNSET))

        ignore_alert_actions_from_payload = d.pop("ignoreAlertActionsFromPayload", UNSET)

        recipients = []
        _recipients = d.pop("recipients", UNSET)
        for recipients_item_data in _recipients or []:
            recipients_item = Recipient.from_dict(recipients_item_data)

            recipients.append(recipients_item)

        responders = []
        _responders = d.pop("responders", UNSET)
        for responders_item_data in _responders or []:
            responders_item = Recipient.from_dict(responders_item_data)

            responders.append(responders_item)

        ignore_recipients_from_payload = d.pop("ignoreRecipientsFromPayload", UNSET)

        ignore_teams_from_payload = d.pop("ignoreTeamsFromPayload", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        ignore_tags_from_payload = d.pop("ignoreTagsFromPayload", UNSET)

        _extra_properties = d.pop("extraProperties", UNSET)
        extra_properties: Union[Unset, CreateIntegrationActionExtraProperties]
        if isinstance(_extra_properties, Unset):
            extra_properties = UNSET
        else:
            extra_properties = CreateIntegrationActionExtraProperties.from_dict(_extra_properties)

        ignore_extra_properties_from_payload = d.pop("ignoreExtraPropertiesFromPayload", UNSET)

        ignore_responders_from_payload = d.pop("ignoreRespondersFromPayload", UNSET)

        create_integration_action = cls(
            name=name,
            filter_=filter_,
            type=type,
            order=order,
            user=user,
            note=note,
            alias=alias,
            source=source,
            message=message,
            description=description,
            entity=entity,
            priority=priority,
            custom_priority=custom_priority,
            append_attachments=append_attachments,
            alert_actions=alert_actions,
            ignore_alert_actions_from_payload=ignore_alert_actions_from_payload,
            recipients=recipients,
            responders=responders,
            ignore_recipients_from_payload=ignore_recipients_from_payload,
            ignore_teams_from_payload=ignore_teams_from_payload,
            tags=tags,
            ignore_tags_from_payload=ignore_tags_from_payload,
            extra_properties=extra_properties,
            ignore_extra_properties_from_payload=ignore_extra_properties_from_payload,
            ignore_responders_from_payload=ignore_responders_from_payload,
        )

        create_integration_action.additional_properties = d
        return create_integration_action

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="AlertRequestStatus")


@_attrs_define
class AlertRequestStatus:
    """
    Attributes:
        action (Union[Unset, str]):
        processed_at (Union[Unset, datetime.datetime]):
        integration_id (Union[Unset, str]):
        is_success (Union[Unset, bool]):
        status (Union[Unset, str]):
        alert_id (Union[Unset, str]):
        alias (Union[Unset, str]):
    """

    action: Union[Unset, str] = UNSET
    processed_at: Union[Unset, datetime.datetime] = UNSET
    integration_id: Union[Unset, str] = UNSET
    is_success: Union[Unset, bool] = UNSET
    status: Union[Unset, str] = UNSET
    alert_id: Union[Unset, str] = UNSET
    alias: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        action = self.action

        processed_at: Union[Unset, str] = UNSET
        if not isinstance(self.processed_at, Unset):
            processed_at = self.processed_at.isoformat()

        integration_id = self.integration_id

        is_success = self.is_success

        status = self.status

        alert_id = self.alert_id

        alias = self.alias

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if action is not UNSET:
            field_dict["action"] = action
        if processed_at is not UNSET:
            field_dict["processedAt"] = processed_at
        if integration_id is not UNSET:
            field_dict["integrationId"] = integration_id
        if is_success is not UNSET:
            field_dict["isSuccess"] = is_success
        if status is not UNSET:
            field_dict["status"] = status
        if alert_id is not UNSET:
            field_dict["alertId"] = alert_id
        if alias is not UNSET:
            field_dict["alias"] = alias

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        action = d.pop("action", UNSET)

        _processed_at = d.pop("processedAt", UNSET)
        processed_at: Union[Unset, datetime.datetime]
        if isinstance(_processed_at, Unset):
            processed_at = UNSET
        else:
            processed_at = isoparse(_processed_at)

        integration_id = d.pop("integrationId", UNSET)

        is_success = d.pop("isSuccess", UNSET)

        status = d.pop("status", UNSET)

        alert_id = d.pop("alertId", UNSET)

        alias = d.pop("alias", UNSET)

        alert_request_status = cls(
            action=action,
            processed_at=processed_at,
            integration_id=integration_id,
            is_success=is_success,
            status=status,
            alert_id=alert_id,
            alias=alias,
        )

        alert_request_status.additional_properties = d
        return alert_request_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.duration_time_unit import DurationTimeUnit
from ..types import UNSET, Unset

T = TypeVar("T", bound="Duration")


@_attrs_define
class Duration:
    """
    Attributes:
        time_amount (int):
        time_unit (Union[Unset, DurationTimeUnit]):  Default: DurationTimeUnit.MINUTES.
    """

    time_amount: int
    time_unit: Union[Unset, DurationTimeUnit] = DurationTimeUnit.MINUTES
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        time_amount = self.time_amount

        time_unit: Union[Unset, str] = UNSET
        if not isinstance(self.time_unit, Unset):
            time_unit = self.time_unit.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "timeAmount": time_amount,
            }
        )
        if time_unit is not UNSET:
            field_dict["timeUnit"] = time_unit

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        time_amount = d.pop("timeAmount")

        _time_unit = d.pop("timeUnit", UNSET)
        time_unit: Union[Unset, DurationTimeUnit]
        if isinstance(_time_unit, Unset):
            time_unit = UNSET
        else:
            time_unit = DurationTimeUnit(_time_unit)

        duration = cls(
            time_amount=time_amount,
            time_unit=time_unit,
        )

        duration.additional_properties = d
        return duration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="UpdateUserPayloadDetails")


@_attrs_define
class UpdateUserPayloadDetails:
    """Set of user defined properties."""

    additional_properties: Dict[str, List[str]] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        update_user_payload_details = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = cast(List[str], prop_dict)

            additional_properties[prop_name] = additional_property

        update_user_payload_details.additional_properties = additional_properties
        return update_user_payload_details

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> List[str]:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: List[str]) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.service_audience_template_condition_match_field import ServiceAudienceTemplateConditionMatchField
from ..types import UNSET, Unset

T = TypeVar("T", bound="ServiceAudienceTemplateCondition")


@_attrs_define
class ServiceAudienceTemplateCondition:
    """
    Attributes:
        match_field (ServiceAudienceTemplateConditionMatchField): Field to be matched for users. Possible values are
            [country, state, city, zipCode, line, tag, customProperty]. customProperty can be used while actionType is
            keyValue.
        value (str): Value to be check for the match field.
        key (Union[Unset, str]): If matchField is customProperty, key must be given.
    """

    match_field: ServiceAudienceTemplateConditionMatchField
    value: str
    key: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        match_field = self.match_field.value

        value = self.value

        key = self.key

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "matchField": match_field,
                "value": value,
            }
        )
        if key is not UNSET:
            field_dict["key"] = key

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        match_field = ServiceAudienceTemplateConditionMatchField(d.pop("matchField"))

        value = d.pop("value")

        key = d.pop("key", UNSET)

        service_audience_template_condition = cls(
            match_field=match_field,
            value=value,
            key=key,
        )

        service_audience_template_condition.additional_properties = d
        return service_audience_template_condition

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from textual.widget import Widget


class CenterMiddle(Widget, inherit_bindings=False):
    """A container which aligns children on both axes."""

    DEFAULT_CSS = """
    CenterMiddle {
        align: center middle;
        width: 1fr;
        height: 1fr;
    }
    """



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.filter_type import FilterType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.condition import Condition


T = TypeVar("T", bound="MatchAllConditions")


@_attrs_define
class MatchAllConditions:
    """Match all conditions filter

    Attributes:
        type (FilterType):
        conditions (Union[Unset, List['Condition']]):
    """

    type: FilterType
    conditions: Union[Unset, List["Condition"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        conditions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.conditions, Unset):
            conditions = []
            for conditions_item_data in self.conditions:
                conditions_item = conditions_item_data.to_dict()
                conditions.append(conditions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
            }
        )
        if conditions is not UNSET:
            field_dict["conditions"] = conditions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.condition import Condition

        d = src_dict.copy()
        type = FilterType(d.pop("type"))

        conditions = []
        _conditions = d.pop("conditions", UNSET)
        for conditions_item_data in _conditions or []:
            conditions_item = Condition.from_dict(conditions_item_data)

            conditions.append(conditions_item)

        match_all_conditions = cls(
            type=type,
            conditions=conditions,
        )

        match_all_conditions.additional_properties = d
        return match_all_conditions

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.snooze_alert_identifier_type import SnoozeAlertIdentifierType
from ...models.snooze_alert_payload import SnoozeAlertPayload
from ...types import UNSET, Response, Unset


def _get_kwargs(
    identifier: str,
    *,
    body: SnoozeAlertPayload,
    identifier_type: Union[Unset, SnoozeAlertIdentifierType] = SnoozeAlertIdentifierType.ID,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    params: Dict[str, Any] = {}

    json_identifier_type: Union[Unset, str] = UNSET
    if not isinstance(identifier_type, Unset):
        json_identifier_type = identifier_type.value

    params["identifierType"] = json_identifier_type

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": f"/v2/alerts/{identifier}/snooze",
        "params": params,
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json;charset=UTF-8"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == 202:
        return None
    if response.status_code == 400:
        return None
    if response.status_code == 401:
        return None
    if response.status_code == 402:
        return None
    if response.status_code == 403:
        return None
    if response.status_code == 404:
        return None
    if response.status_code == 422:
        return None
    if response.status_code == 429:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    identifier: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: SnoozeAlertPayload,
    identifier_type: Union[Unset, SnoozeAlertIdentifierType] = SnoozeAlertIdentifierType.ID,
) -> Response[Any]:
    """Snooze Alert

     Snooze alert with given identifier

    Args:
        identifier (str):
        identifier_type (Union[Unset, SnoozeAlertIdentifierType]):  Default:
            SnoozeAlertIdentifierType.ID.
        body (SnoozeAlertPayload):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        identifier=identifier,
        body=body,
        identifier_type=identifier_type,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    identifier: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: SnoozeAlertPayload,
    identifier_type: Union[Unset, SnoozeAlertIdentifierType] = SnoozeAlertIdentifierType.ID,
) -> Response[Any]:
    """Snooze Alert

     Snooze alert with given identifier

    Args:
        identifier (str):
        identifier_type (Union[Unset, SnoozeAlertIdentifierType]):  Default:
            SnoozeAlertIdentifierType.ID.
        body (SnoozeAlertPayload):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        identifier=identifier,
        body=body,
        identifier_type=identifier_type,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.escalation_rule_condition import EscalationRuleCondition
from ..models.escalation_rule_notify_type import EscalationRuleNotifyType

if TYPE_CHECKING:
    from ..models.duration import Duration
    from ..models.recipient import Recipient


T = TypeVar("T", bound="EscalationRule")


@_attrs_define
class EscalationRule:
    """
    Attributes:
        condition (EscalationRuleCondition):  Default: EscalationRuleCondition.IF_NOT_ACKED.
        notify_type (EscalationRuleNotifyType):  Default: EscalationRuleNotifyType.DEFAULT.
        delay (Duration):
        recipient (Recipient):
    """

    delay: "Duration"
    recipient: "Recipient"
    condition: EscalationRuleCondition = EscalationRuleCondition.IF_NOT_ACKED
    notify_type: EscalationRuleNotifyType = EscalationRuleNotifyType.DEFAULT
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        condition = self.condition.value

        notify_type = self.notify_type.value

        delay = self.delay.to_dict()

        recipient = self.recipient.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "condition": condition,
                "notifyType": notify_type,
                "delay": delay,
                "recipient": recipient,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.duration import Duration
        from ..models.recipient import Recipient

        d = src_dict.copy()
        condition = EscalationRuleCondition(d.pop("condition"))

        notify_type = EscalationRuleNotifyType(d.pop("notifyType"))

        delay = Duration.from_dict(d.pop("delay"))

        recipient = Recipient.from_dict(d.pop("recipient"))

        escalation_rule = cls(
            condition=condition,
            notify_type=notify_type,
            delay=delay,
            recipient=recipient,
        )

        escalation_rule.additional_properties = d
        return escalation_rule

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.participant import Participant
    from ..models.schedule_meta import ScheduleMeta


T = TypeVar("T", bound="NextOnCall")


@_attrs_define
class NextOnCall:
    """
    Attributes:
        field_parent (Union[Unset, ScheduleMeta]):
        next_on_call_recipients (Union[Unset, List['Participant']]):
        exact_next_on_call_recipients (Union[Unset, List['Participant']]):
        next_on_call_participants (Union[Unset, List[str]]):
        exact_next_on_call_participants (Union[Unset, List[str]]):
    """

    field_parent: Union[Unset, "ScheduleMeta"] = UNSET
    next_on_call_recipients: Union[Unset, List["Participant"]] = UNSET
    exact_next_on_call_recipients: Union[Unset, List["Participant"]] = UNSET
    next_on_call_participants: Union[Unset, List[str]] = UNSET
    exact_next_on_call_participants: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field_parent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.field_parent, Unset):
            field_parent = self.field_parent.to_dict()

        next_on_call_recipients: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.next_on_call_recipients, Unset):
            next_on_call_recipients = []
            for next_on_call_recipients_item_data in self.next_on_call_recipients:
                next_on_call_recipients_item = next_on_call_recipients_item_data.to_dict()
                next_on_call_recipients.append(next_on_call_recipients_item)

        exact_next_on_call_recipients: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.exact_next_on_call_recipients, Unset):
            exact_next_on_call_recipients = []
            for exact_next_on_call_recipients_item_data in self.exact_next_on_call_recipients:
                exact_next_on_call_recipients_item = exact_next_on_call_recipients_item_data.to_dict()
                exact_next_on_call_recipients.append(exact_next_on_call_recipients_item)

        next_on_call_participants: Union[Unset, List[str]] = UNSET
        if not isinstance(self.next_on_call_participants, Unset):
            next_on_call_participants = self.next_on_call_participants

        exact_next_on_call_participants: Union[Unset, List[str]] = UNSET
        if not isinstance(self.exact_next_on_call_participants, Unset):
            exact_next_on_call_participants = self.exact_next_on_call_participants

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if field_parent is not UNSET:
            field_dict["_parent"] = field_parent
        if next_on_call_recipients is not UNSET:
            field_dict["nextOnCallRecipients"] = next_on_call_recipients
        if exact_next_on_call_recipients is not UNSET:
            field_dict["exactNextOnCallRecipients"] = exact_next_on_call_recipients
        if next_on_call_participants is not UNSET:
            field_dict["nextOnCallParticipants"] = next_on_call_participants
        if exact_next_on_call_participants is not UNSET:
            field_dict["exactNextOnCallParticipants"] = exact_next_on_call_participants

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.participant import Participant
        from ..models.schedule_meta import ScheduleMeta

        d = src_dict.copy()
        _field_parent = d.pop("_parent", UNSET)
        field_parent: Union[Unset, ScheduleMeta]
        if isinstance(_field_parent, Unset):
            field_parent = UNSET
        else:
            field_parent = ScheduleMeta.from_dict(_field_parent)

        next_on_call_recipients = []
        _next_on_call_recipients = d.pop("nextOnCallRecipients", UNSET)
        for next_on_call_recipients_item_data in _next_on_call_recipients or []:
            next_on_call_recipients_item = Participant.from_dict(next_on_call_recipients_item_data)

            next_on_call_recipients.append(next_on_call_recipients_item)

        exact_next_on_call_recipients = []
        _exact_next_on_call_recipients = d.pop("exactNextOnCallRecipients", UNSET)
        for exact_next_on_call_recipients_item_data in _exact_next_on_call_recipients or []:
            exact_next_on_call_recipients_item = Participant.from_dict(exact_next_on_call_recipients_item_data)

            exact_next_on_call_recipients.append(exact_next_on_call_recipients_item)

        next_on_call_participants = cast(List[str], d.pop("nextOnCallParticipants", UNSET))

        exact_next_on_call_participants = cast(List[str], d.pop("exactNextOnCallParticipants", UNSET))

        next_on_call = cls(
            field_parent=field_parent,
            next_on_call_recipients=next_on_call_recipients,
            exact_next_on_call_recipients=exact_next_on_call_recipients,
            next_on_call_participants=next_on_call_participants,
            exact_next_on_call_participants=exact_next_on_call_participants,
        )

        next_on_call.additional_properties = d
        return next_on_call

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from textual.app import ComposeResult
from textual.containers import Horizontal
from textual.reactive import reactive
from textual.widget import Widget

from tygenie.widgets.button import TyButtonOpenInWebBrowser


class Buttons(Widget):

    names = reactive("names", recompose=True)

    def __init__(self, names: list[tuple] = [], **kwargs):
        super().__init__(**kwargs)
        self.names_urls = names

    def compose(self) -> ComposeResult:
        with Horizontal():
            for name, _ in self.names_urls:
                yield TyButtonOpenInWebBrowser(label=name, name=name)

    def update(self, names_urls: list[tuple]):
        self.names_urls = names_urls



from enum import Enum


class BidirectionalCallbackNewBidirectionalCallbackType(str, Enum):
    BMC_FOOT_PRINTS_V11_CALLBACK = "bmc-foot-prints-v11-callback"
    BMC_FOOT_PRINTS_V12_CALLBACK = "bmc-foot-prints-v12-callback"
    BMC_REMEDY_CALLBACK = "bmc-remedy-callback"
    CHERWELL_CALLBACK = "cherwell-callback"
    CIRCONUS_CALLBACK = "circonus-callback"
    CONNECT_WISE_MANAGE_CALLBACK = "connect-wise-manage-callback"
    CONNECT_WISE_MANAGE_V2_CALLBACK = "connect-wise-manage-v2-callback"
    DATADOG_CALLBACK = "datadog-callback"
    DYNATRACE_APP_MON_CALLBACK = "dynatrace-app-mon-callback"
    FRESHDESK_CALLBACK = "freshdesk-callback"
    FRESHSERVICE_CALLBACK = "freshservice-callback"
    HP_SERVICE_MANAGER_CALLBACK = "hp-service-manager-callback"
    JIRA_CALLBACK = "jira-callback"
    JIRA_SERVICE_DESK_CALLBACK = "jira-service-desk-callback"
    KAYAKO_CALLBACK = "kayako-callback"
    LIBRE_NMS_CALLBACK = "libre-nms-callback"
    LOGIC_MONITOR_CALLBACK = "logic-monitor-callback"
    MAGENTRIX_CALLBACK = "magentrix-callback"
    MS_TEAMS_CALLBACK = "ms-teams-callback"
    MS_TEAMS_V2_CALLBACK = "ms-teams-v2-callback"
    OP5_CALLBACK = "op5-callback"
    OPS_GENIE_CALLBACK = "ops-genie-callback"
    PRTG_CALLBACK = "prtg-callback"
    ROLLBAR_CALLBACK = "rollbar-callback"
    SALES_FORCE_SERVICE_CLOUD_CALLBACK = "sales-force-service-cloud-callback"
    SERVICE_NOW_V2_CALLBACK = "service-now-v2-callback"
    SERVICE_NOW_V3_CALLBACK = "service-now-v3-callback"
    SOLARWINDS_MSP_NCENTRAL_CALLBACK = "solarwinds-msp-ncentral-callback"
    SPLUNK_CALLBACK = "splunk-callback"
    SPLUNK_ITSI_CALLBACK = "splunk-itsi-callback"
    STATUS_PAGE_IO_CALLBACK = "status-page-io-callback"
    SUMO_LOGIC_CALLBACK = "sumo-logic-callback"
    ZENDESK_CALLBACK = "zendesk-callback"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.condition_field import ConditionField
from ..models.condition_operation import ConditionOperation
from ..types import UNSET, Unset

T = TypeVar("T", bound="Condition")


@_attrs_define
class Condition:
    """
    Attributes:
        field (ConditionField): Specifies which alert field will be used in condition. Possible values are message,
            alias, description, source, entity, tags, actions, extra-properties, recipients or teams
        operation (ConditionOperation): It is the operation that will be executed for the given field and key.
        key (Union[Unset, str]): If field is set as extra-properties, key could be used for key-value pair
        not_ (Union[Unset, bool]): Indicates behaviour of the given operation. Default value is false
        expected_value (Union[Unset, str]): User defined value that will be compared with alert field according to the
            operation. Default value is empty string
        order (Union[Unset, int]): Order of the condition in conditions list
    """

    field: ConditionField
    operation: ConditionOperation
    key: Union[Unset, str] = UNSET
    not_: Union[Unset, bool] = UNSET
    expected_value: Union[Unset, str] = UNSET
    order: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field = self.field.value

        operation = self.operation.value

        key = self.key

        not_ = self.not_

        expected_value = self.expected_value

        order = self.order

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "field": field,
                "operation": operation,
            }
        )
        if key is not UNSET:
            field_dict["key"] = key
        if not_ is not UNSET:
            field_dict["not"] = not_
        if expected_value is not UNSET:
            field_dict["expectedValue"] = expected_value
        if order is not UNSET:
            field_dict["order"] = order

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        field = ConditionField(d.pop("field"))

        operation = ConditionOperation(d.pop("operation"))

        key = d.pop("key", UNSET)

        not_ = d.pop("not", UNSET)

        expected_value = d.pop("expectedValue", UNSET)

        order = d.pop("order", UNSET)

        condition = cls(
            field=field,
            operation=operation,
            key=key,
            not_=not_,
            expected_value=expected_value,
            order=order,
        )

        condition.additional_properties = d
        return condition

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.base_integration_action_type import BaseIntegrationActionType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.integration_action_filter import IntegrationActionFilter


T = TypeVar("T", bound="CommonIntegrationAction")


@_attrs_define
class CommonIntegrationAction:
    """
    Attributes:
        name (str):
        filter_ (IntegrationActionFilter):
        type (BaseIntegrationActionType):
        order (Union[Unset, int]):
        user (Union[Unset, str]):
        note (Union[Unset, str]):
        alias (Union[Unset, str]):
    """

    name: str
    filter_: "IntegrationActionFilter"
    type: BaseIntegrationActionType
    order: Union[Unset, int] = UNSET
    user: Union[Unset, str] = UNSET
    note: Union[Unset, str] = UNSET
    alias: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        filter_ = self.filter_.to_dict()

        type = self.type.value

        order = self.order

        user = self.user

        note = self.note

        alias = self.alias

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "filter": filter_,
                "type": type,
            }
        )
        if order is not UNSET:
            field_dict["order"] = order
        if user is not UNSET:
            field_dict["user"] = user
        if note is not UNSET:
            field_dict["note"] = note
        if alias is not UNSET:
            field_dict["alias"] = alias

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.integration_action_filter import IntegrationActionFilter

        d = src_dict.copy()
        name = d.pop("name")

        filter_ = IntegrationActionFilter.from_dict(d.pop("filter"))

        type = BaseIntegrationActionType(d.pop("type"))

        order = d.pop("order", UNSET)

        user = d.pop("user", UNSET)

        note = d.pop("note", UNSET)

        alias = d.pop("alias", UNSET)

        common_integration_action = cls(
            name=name,
            filter_=filter_,
            type=type,
            order=order,
            user=user,
            note=note,
            alias=alias,
        )

        common_integration_action.additional_properties = d
        return common_integration_action

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.schedule_rotation_type import ScheduleRotationType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.recipient import Recipient
    from ..models.time_restriction_interval import TimeRestrictionInterval


T = TypeVar("T", bound="ScheduleRotation")


@_attrs_define
class ScheduleRotation:
    """
    Attributes:
        id (Union[Unset, str]):
        name (Union[Unset, str]):
        start_date (Union[Unset, datetime.datetime]):
        end_date (Union[Unset, datetime.datetime]):
        type (Union[Unset, ScheduleRotationType]):
        length (Union[Unset, int]):
        participants (Union[Unset, List['Recipient']]):
        time_restriction (Union[Unset, TimeRestrictionInterval]):
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    start_date: Union[Unset, datetime.datetime] = UNSET
    end_date: Union[Unset, datetime.datetime] = UNSET
    type: Union[Unset, ScheduleRotationType] = UNSET
    length: Union[Unset, int] = UNSET
    participants: Union[Unset, List["Recipient"]] = UNSET
    time_restriction: Union[Unset, "TimeRestrictionInterval"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        length = self.length

        participants: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.participants, Unset):
            participants = []
            for participants_item_data in self.participants:
                participants_item = participants_item_data.to_dict()
                participants.append(participants_item)

        time_restriction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.time_restriction, Unset):
            time_restriction = self.time_restriction.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if type is not UNSET:
            field_dict["type"] = type
        if length is not UNSET:
            field_dict["length"] = length
        if participants is not UNSET:
            field_dict["participants"] = participants
        if time_restriction is not UNSET:
            field_dict["timeRestriction"] = time_restriction

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.recipient import Recipient
        from ..models.time_restriction_interval import TimeRestrictionInterval

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.datetime]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date)

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.datetime]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date)

        _type = d.pop("type", UNSET)
        type: Union[Unset, ScheduleRotationType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = ScheduleRotationType(_type)

        length = d.pop("length", UNSET)

        participants = []
        _participants = d.pop("participants", UNSET)
        for participants_item_data in _participants or []:
            participants_item = Recipient.from_dict(participants_item_data)

            participants.append(participants_item)

        _time_restriction = d.pop("timeRestriction", UNSET)
        time_restriction: Union[Unset, TimeRestrictionInterval]
        if isinstance(_time_restriction, Unset):
            time_restriction = UNSET
        else:
            time_restriction = TimeRestrictionInterval.from_dict(_time_restriction)

        schedule_rotation = cls(
            id=id,
            name=name,
            start_date=start_date,
            end_date=end_date,
            type=type,
            length=length,
            participants=participants,
            time_restriction=time_restriction,
        )

        schedule_rotation.additional_properties = d
        return schedule_rotation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class CreateServicePayloadVisibility(str, Enum):
    OPSGENIE_USERS = "OPSGENIE_USERS"
    TEAM_MEMBERS = "TEAM_MEMBERS"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.integration_action_filter_condition_match_type import IntegrationActionFilterConditionMatchType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.integration_condition import IntegrationCondition


T = TypeVar("T", bound="IntegrationActionFilter")


@_attrs_define
class IntegrationActionFilter:
    """
    Attributes:
        condition_match_type (Union[Unset, IntegrationActionFilterConditionMatchType]):
        conditions (Union[Unset, List['IntegrationCondition']]):
    """

    condition_match_type: Union[Unset, IntegrationActionFilterConditionMatchType] = UNSET
    conditions: Union[Unset, List["IntegrationCondition"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        condition_match_type: Union[Unset, str] = UNSET
        if not isinstance(self.condition_match_type, Unset):
            condition_match_type = self.condition_match_type.value

        conditions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.conditions, Unset):
            conditions = []
            for conditions_item_data in self.conditions:
                conditions_item = conditions_item_data.to_dict()
                conditions.append(conditions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if condition_match_type is not UNSET:
            field_dict["conditionMatchType"] = condition_match_type
        if conditions is not UNSET:
            field_dict["conditions"] = conditions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.integration_condition import IntegrationCondition

        d = src_dict.copy()
        _condition_match_type = d.pop("conditionMatchType", UNSET)
        condition_match_type: Union[Unset, IntegrationActionFilterConditionMatchType]
        if isinstance(_condition_match_type, Unset):
            condition_match_type = UNSET
        else:
            condition_match_type = IntegrationActionFilterConditionMatchType(_condition_match_type)

        conditions = []
        _conditions = d.pop("conditions", UNSET)
        for conditions_item_data in _conditions or []:
            conditions_item = IntegrationCondition.from_dict(conditions_item_data)

            conditions.append(conditions_item)

        integration_action_filter = cls(
            condition_match_type=condition_match_type,
            conditions=conditions,
        )

        integration_action_filter.additional_properties = d
        return integration_action_filter

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import asyncio

import pendulum
from httpx import Timeout

import tygenie.config as config
import tygenie.consts as c
import tygenie.logger as ty_logger
from tygenie.opsgenie_rest_api_client import AuthenticatedClient
from tygenie.opsgenie_rest_api_client.api.account import get_info
from tygenie.opsgenie_rest_api_client.api.alert import (
    acknowledge_alert,
    add_note,
    add_tags,
    close_alert,
    count_alerts,
    get_alert,
    list_alerts,
    list_notes,
    remove_tags,
    un_acknowledge_alert,
)
from tygenie.opsgenie_rest_api_client.api.schedule import list_schedules
from tygenie.opsgenie_rest_api_client.api.who_is_on_call import (
    get_on_calls,
)
from tygenie.opsgenie_rest_api_client.models.add_tags_to_alert_payload import (
    AddTagsToAlertPayload,
)
from tygenie.opsgenie_rest_api_client.models.alert_action_payload import (
    AlertActionPayload,
)

__all__ = (
    "OpsGenie",
    "client",
)


class OpsGenie:

    def __init__(self, api_key: str = "", host: str = "", username: str = ""):
        self.api_key = api_key
        self.ost = host
        self.username = username
        self._load_config()
        self.client = AuthenticatedClient(
            base_url=self.host,
            token=self.api_key,
            prefix="GenieKey",
            timeout=Timeout(5.0, connect=10.0),
        )

    async def get_account_info(self):
        return await self.api_call(get_info)

    async def count_alerts(self, parameters: dict = {}):
        params = {"query": parameters.get("query", "")}
        return await self.api_call(count_alerts, **params)

    async def list_alerts(self, limit: int = 50, parameters: dict = {}):
        params = {"limit": limit, "sort": "updatedAt", "order": "desc", "query": ""}
        params.update(parameters)
        return await self.api_call(list_alerts, **params)

    async def get_alert(self, parameters: dict = {}):
        return await self.api_call(get_alert, **parameters)

    async def get_alert_notes(self, parameters: dict = {}):
        return await self.api_call(list_notes, **parameters)

    async def ack_alert(self, parameters: dict = {}, note: str = ""):
        body = AlertActionPayload(user=self.username, source=self.source, note=note)
        parameters["body"] = body
        return await self.api_call(acknowledge_alert, **parameters)

    async def add_note(self, parameters: dict = {}, note: str = ""):
        body = AlertActionPayload(user=self.username, source=self.source, note=note)
        parameters["body"] = body
        return await self.api_call(add_note, **parameters)

    async def unack_alert(self, parameters: dict = {}, note: str = ""):
        body = AlertActionPayload(user=self.username, source=self.source, note=note)
        parameters["body"] = body
        return await self.api_call(un_acknowledge_alert, **parameters)

    async def close_alert(self, parameters: dict = {}, note: str = ""):
        body = AlertActionPayload(
            user=self.username, source="TyGenie {}".format(c.VERSION), note=note
        )
        parameters["body"] = body
        return await self.api_call(close_alert, **parameters)

    async def tag_alert(
        self, parameters: dict = {}, tags: list[str] = [], note: str = ""
    ):
        body = AddTagsToAlertPayload(
            user=self.username, source=self.source, note=note, tags=tags
        )
        parameters["body"] = body
        return await self.api_call(add_tags, **parameters)

    async def remove_tag_alert(
        self, parameters: dict = {}, tags: list[str] = [], note: str = ""
    ):
        # There is no RemoveTagsToAlertPayload
        params = {
            "user": self.username,
            "source": self.source,
            "tags": tags,
            "note": note,
            "identifier": parameters["identifier"],
        }
        return await self.api_call(remove_tags, **params)

    async def list_schedules(self):
        return await self.api_call(list_schedules)

    async def whois_on_call(self, parameters: dict = {}):
        params = {"flat": True, "date": pendulum.now()} | parameters
        return await self.api_call(get_on_calls, **params)

    async def api_call(self, resource, **kwargs):

        response = None
        try:
            ty_logger.logger.log(f"API call {resource.__name__} with params {kwargs}")
            response = await getattr(resource, "asyncio_detailed")(
                client=self.client, **kwargs
            )
            ty_logger.logger.log(f"API status code: {response.status_code}")
            ty_logger.logger.log(f"API content: {response.content}")
            ty_logger.logger.log(f"API response headers: {response.headers}")
            ty_logger.logger.log(f"API call {resource.__name__} done")
            return response.parsed
        except Exception as e:
            ty_logger.logger.log(f"Exception in API call: {e}")
            return response

    def _load_config(self):
        self.api_key = config.ty_config.opsgenie.get("api_key", "")
        self.host = config.ty_config.opsgenie.get("host", "")
        self.username = config.ty_config.opsgenie.get("username", "")
        self.source = "TyGenie {}".format(c.VERSION)

    def load(self):
        self._load_config()

    def reload(self):
        self._load_config()


class Query:

    def __init__(self) -> None:
        self._limit: int = 22
        self.sort: str = "createdAt"
        self.order: str = "desc"
        self.query: str = "status:open"
        self.offset: int = 0
        self.current_filter: str | None = None
        self.current: dict = self.get()

    @property
    def limit(self) -> int:
        self._limit: int = int(config.ty_config.tygenie["alerts"].get("limit", 22))
        return self._limit

    @limit.setter
    def limit(self, value=0) -> int:
        self._limit: int = value or int(
            config.ty_config.tygenie["alerts"].get("limit", 22)
        )
        return self._limit

    def _get_query(self, filter_name: str | None = None) -> str:
        query: str = ""
        if filter_name is None:
            if self.current_filter is not None:
                filter_name = self.current_filter
            else:
                filter_name = config.ty_config.tygenie.get("default_filter", None)

        if filter_name is not None:
            filters: dict = config.ty_config.tygenie.get("filters", {})
            cust_filter: dict | None = filters.get(filter_name, None)
            if cust_filter is None:
                ty_logger.logger.log(f"Custom filter '{filter_name}' not found")
            else:
                query = cust_filter.get("filter", "")

        self.current_filter = filter_name

        return query

    def get(self, filter_name: str | None = None, parameters: dict = {}) -> dict:
        query: str = self._get_query(filter_name=filter_name)
        params: dict = {
            "limit": self.limit,
            "sort": self.sort,
            "order": self.order,
            "offset": self.offset,
            "query": query,
        }
        return params | parameters

    def current_page(self) -> int:
        return int(self.offset / self.limit) + 1

    def get_next(self) -> dict:
        self.offset += self.limit
        return self.get(parameters={"offset": self.offset})

    def get_previous(self) -> dict:
        self.offset -= self.limit
        return self.get(parameters={"offset": max(0, self.offset)})


class OpsgenieClient(OpsGenie):

    def __init__(self) -> None:
        self.api: OpsGenie = OpsGenie()
        self._load()
        super().__init__()

    def _load(self) -> None:
        self.api.load()

    def reload(self) -> None:
        self._load()


client = OpsgenieClient()


def reload():
    global client
    client = OpsgenieClient()


if __name__ == "__main__":

    async def main():
        task = asyncio.create_task(client.api.count_alerts())
        await task

    task = asyncio.run(main())
    print(task)



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user_address import UserAddress
    from ..models.user_contact import UserContact
    from ..models.user_details import UserDetails
    from ..models.user_role import UserRole


T = TypeVar("T", bound="User")


@_attrs_define
class User:
    """
    Attributes:
        id (Union[Unset, str]):
        username (Union[Unset, str]):
        full_name (Union[Unset, str]):
        role (Union[Unset, UserRole]):
        skype_username (Union[Unset, str]): Skype username of the user
        time_zone (Union[Unset, str]): Timezone of the user. If not set, timezone of the customer will be used instead.
        locale (Union[Unset, str]): Location information of the user. If not set, locale of the customer will be used
            instead.
        user_address (Union[Unset, UserAddress]):
        tags (Union[Unset, List[str]]): List of labels attached to the user. You can label users to differentiate them
            from the rest. For example, you can add ITManager tag to differentiate people with this role from others.
        details (Union[Unset, UserDetails]): Set of user defined properties.
        blocked (Union[Unset, bool]):
        verified (Union[Unset, bool]):
        created_at (Union[Unset, datetime.datetime]):
        muted_until (Union[Unset, datetime.datetime]):
        user_contacts (Union[Unset, List['UserContact']]):
    """

    id: Union[Unset, str] = UNSET
    username: Union[Unset, str] = UNSET
    full_name: Union[Unset, str] = UNSET
    role: Union[Unset, "UserRole"] = UNSET
    skype_username: Union[Unset, str] = UNSET
    time_zone: Union[Unset, str] = UNSET
    locale: Union[Unset, str] = UNSET
    user_address: Union[Unset, "UserAddress"] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    details: Union[Unset, "UserDetails"] = UNSET
    blocked: Union[Unset, bool] = UNSET
    verified: Union[Unset, bool] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    muted_until: Union[Unset, datetime.datetime] = UNSET
    user_contacts: Union[Unset, List["UserContact"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        username = self.username

        full_name = self.full_name

        role: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.to_dict()

        skype_username = self.skype_username

        time_zone = self.time_zone

        locale = self.locale

        user_address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.user_address, Unset):
            user_address = self.user_address.to_dict()

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.details, Unset):
            details = self.details.to_dict()

        blocked = self.blocked

        verified = self.verified

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        muted_until: Union[Unset, str] = UNSET
        if not isinstance(self.muted_until, Unset):
            muted_until = self.muted_until.isoformat()

        user_contacts: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.user_contacts, Unset):
            user_contacts = []
            for user_contacts_item_data in self.user_contacts:
                user_contacts_item = user_contacts_item_data.to_dict()
                user_contacts.append(user_contacts_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if username is not UNSET:
            field_dict["username"] = username
        if full_name is not UNSET:
            field_dict["fullName"] = full_name
        if role is not UNSET:
            field_dict["role"] = role
        if skype_username is not UNSET:
            field_dict["skypeUsername"] = skype_username
        if time_zone is not UNSET:
            field_dict["timeZone"] = time_zone
        if locale is not UNSET:
            field_dict["locale"] = locale
        if user_address is not UNSET:
            field_dict["userAddress"] = user_address
        if tags is not UNSET:
            field_dict["tags"] = tags
        if details is not UNSET:
            field_dict["details"] = details
        if blocked is not UNSET:
            field_dict["blocked"] = blocked
        if verified is not UNSET:
            field_dict["verified"] = verified
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if muted_until is not UNSET:
            field_dict["mutedUntil"] = muted_until
        if user_contacts is not UNSET:
            field_dict["userContacts"] = user_contacts

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.user_address import UserAddress
        from ..models.user_contact import UserContact
        from ..models.user_details import UserDetails
        from ..models.user_role import UserRole

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        username = d.pop("username", UNSET)

        full_name = d.pop("fullName", UNSET)

        _role = d.pop("role", UNSET)
        role: Union[Unset, UserRole]
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = UserRole.from_dict(_role)

        skype_username = d.pop("skypeUsername", UNSET)

        time_zone = d.pop("timeZone", UNSET)

        locale = d.pop("locale", UNSET)

        _user_address = d.pop("userAddress", UNSET)
        user_address: Union[Unset, UserAddress]
        if isinstance(_user_address, Unset):
            user_address = UNSET
        else:
            user_address = UserAddress.from_dict(_user_address)

        tags = cast(List[str], d.pop("tags", UNSET))

        _details = d.pop("details", UNSET)
        details: Union[Unset, UserDetails]
        if isinstance(_details, Unset):
            details = UNSET
        else:
            details = UserDetails.from_dict(_details)

        blocked = d.pop("blocked", UNSET)

        verified = d.pop("verified", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _muted_until = d.pop("mutedUntil", UNSET)
        muted_until: Union[Unset, datetime.datetime]
        if isinstance(_muted_until, Unset):
            muted_until = UNSET
        else:
            muted_until = isoparse(_muted_until)

        user_contacts = []
        _user_contacts = d.pop("userContacts", UNSET)
        for user_contacts_item_data in _user_contacts or []:
            user_contacts_item = UserContact.from_dict(user_contacts_item_data)

            user_contacts.append(user_contacts_item)

        user = cls(
            id=id,
            username=username,
            full_name=full_name,
            role=role,
            skype_username=skype_username,
            time_zone=time_zone,
            locale=locale,
            user_address=user_address,
            tags=tags,
            details=details,
            blocked=blocked,
            verified=verified,
            created_at=created_at,
            muted_until=muted_until,
            user_contacts=user_contacts,
        )

        user.additional_properties = d
        return user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class MaintenanceTimeType(str, Enum):
    FOR_1_HOUR = "for-1-hour"
    FOR_30_MINUTES = "for-30-minutes"
    FOR_5_MINUTES = "for-5-minutes"
    INDEFINITELY = "indefinitely"
    SCHEDULE = "schedule"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class CreateHeartbeatPayloadIntervalUnit(str, Enum):
    DAYS = "days"
    HOURS = "hours"
    MINUTES = "minutes"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class NotifyTime(str, Enum):
    JUST_BEFORE = "just-before"
    VALUE_1 = "15-minutes-ago"
    VALUE_2 = "1-hour-ago"
    VALUE_3 = "1-day-ago"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.escalation_repeat import EscalationRepeat
    from ..models.escalation_rule import EscalationRule
    from ..models.team_meta import TeamMeta


T = TypeVar("T", bound="CreateEscalationPayload")


@_attrs_define
class CreateEscalationPayload:
    """
    Attributes:
        name (str): Name of the escalation
        rules (List['EscalationRule']): List of escalation rules.
        description (Union[Unset, str]): Description of the escalation
        owner_team (Union[Unset, TeamMeta]):
        repeat (Union[Unset, EscalationRepeat]):
    """

    name: str
    rules: List["EscalationRule"]
    description: Union[Unset, str] = UNSET
    owner_team: Union[Unset, "TeamMeta"] = UNSET
    repeat: Union[Unset, "EscalationRepeat"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        rules = []
        for rules_item_data in self.rules:
            rules_item = rules_item_data.to_dict()
            rules.append(rules_item)

        description = self.description

        owner_team: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner_team, Unset):
            owner_team = self.owner_team.to_dict()

        repeat: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.repeat, Unset):
            repeat = self.repeat.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "rules": rules,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if owner_team is not UNSET:
            field_dict["ownerTeam"] = owner_team
        if repeat is not UNSET:
            field_dict["repeat"] = repeat

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.escalation_repeat import EscalationRepeat
        from ..models.escalation_rule import EscalationRule
        from ..models.team_meta import TeamMeta

        d = src_dict.copy()
        name = d.pop("name")

        rules = []
        _rules = d.pop("rules")
        for rules_item_data in _rules:
            rules_item = EscalationRule.from_dict(rules_item_data)

            rules.append(rules_item)

        description = d.pop("description", UNSET)

        _owner_team = d.pop("ownerTeam", UNSET)
        owner_team: Union[Unset, TeamMeta]
        if isinstance(_owner_team, Unset):
            owner_team = UNSET
        else:
            owner_team = TeamMeta.from_dict(_owner_team)

        _repeat = d.pop("repeat", UNSET)
        repeat: Union[Unset, EscalationRepeat]
        if isinstance(_repeat, Unset):
            repeat = UNSET
        else:
            repeat = EscalationRepeat.from_dict(_repeat)

        create_escalation_payload = cls(
            name=name,
            rules=rules,
            description=description,
            owner_team=owner_team,
            repeat=repeat,
        )

        create_escalation_payload.additional_properties = d
        return create_escalation_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import re
from importlib import import_module

import pendulum
from rich.text import Text
from textual.app import App
from udatetime import from_string as dtfstr

import tygenie.config as config
import tygenie.logger as ty_logger
from tygenie.opsgenie_rest_api_client.models.alert import Alert
from tygenie.opsgenie_rest_api_client.models.alert_report import AlertReport


class BaseFormatter:

    displayed_fields = {
        "created_at": "Created",
        "duration": "Created since",
        "status": "Status",
        "priority": "Priority",
        "message": "Message",
        "owner": "Owner",
        "closed_by": "Closed By",
    }

    colors = {"white": "#ffffff"}

    def __init__(
        self,
        to_format: dict = {},
        formatter_fields: dict = {},
        alert: None | Alert = None,
        disabled: bool = False,
        app: App | None = None,
    ) -> None:
        self.to_format: dict = to_format
        self.formatter_fields: dict = formatter_fields
        self.alert: Alert | None = alert
        self.formatted: dict[Text, Text] = {k: Text("") for k in self.to_format.keys()}
        self.date_format: str = config.ty_config.tygenie.get("alerts", {}).get(
            "date_format", "%d/%m %H:%M"
        )
        self.disabled: bool = disabled
        if isinstance(app, App):
            self.app = app

    def format(self) -> dict:
        for attr, value in self.to_format.items():
            if not hasattr(self, attr) or self.disabled:
                self.formatted[attr] = Text(str(value))
                continue
            self.formatted[attr] = getattr(self, attr)(value)

        return self.formatted

    def _as_date(self, value) -> str:
        if value is None:
            return ""
        # value is a datetime object, so it is safe to call isoformat
        return dtfstr(value.isoformat()).strftime(self.date_format)

    def tiny_id(self, value) -> Text:
        return Text(
            value, style=self.app.theme_variables["secondary"], justify="center"
        )

    def created_at(self, value) -> Text:
        return Text(
            self._as_date(value),
            style=self.app.theme_variables["secondary-lighten-3"],
            justify="center",
        )

    def updated_at(self, value) -> Text:
        return Text(self._as_date(value), style=self.colors["white"], justify="center")

    def last_occurred_at(self, value) -> Text:
        return Text(self._as_date(value), style=self.colors["white"], justify="center")

    def owner(self, value) -> Text:
        return Text(
            re.sub("@.*$", "", value),
            style=self.app.theme_variables["warning"],
            justify="left",
        )

    def light_message(self, value) -> Text:
        return Text(value[0:80])

    def message(self, value) -> Text:
        if len(value) > 100:
            value = value[0:100] + "..."
        return Text(str(value))

    def closed_by(self, value) -> Text:
        value = ""
        if self.alert is not None and isinstance(self.alert.report, AlertReport):
            if self.alert.report.closed_by:
                value = self.alert.report.closed_by

        return Text(
            re.sub("@.*$", "", str(value)),
            style=self.app.theme_variables["warning"],
            justify="left",
        )

    def status(self, value) -> Text:
        if self.alert is None:
            return Text("")

        value = ""

        if self.alert.status == "open":
            value = "open"
            if self.alert.acknowledged:
                value = "acked"
        elif self.alert.status == "closed":
            value = "closed"

        return Text(
            value, style=self.app.theme_variables.get(value, ""), justify="left"
        )

    def priority(self, value) -> Text:
        theme_colors = self.app.theme_variables
        p_colors = [
            theme_colors.get("error", self.colors["white"]),
            theme_colors.get("accent", self.colors["white"]),
            theme_colors.get("warning", self.colors["white"]),
            theme_colors.get("secondary", self.colors["white"]),
            theme_colors.get("primary", self.colors["white"]),
        ]

        match = re.match(r"P(\d+)", value)
        if match is not None:
            return Text(
                value,
                style=p_colors[int(match.groups()[0]) - 1],
                justify="center",
            )
        else:
            return Text("")

    def duration(self, value) -> Text:
        created = pendulum.parse(str(self.formatter_fields["created_at"]), tz="UTC")
        now_utc = pendulum.now(tz="UTC")
        duration = now_utc.diff_for_humans(created, absolute=True)
        return Text(
            duration,
            style=self.app.theme_variables.get("warning", self.colors["white"]),
        )


class AlertFormatter:

    def __init__(
        self,
        alert: Alert | None = None,
        formatter: str | None = None,
        app: App | None = None,
    ) -> None:
        self.alert: Alert | None = alert
        self.formatter: str | None = formatter or None
        self.app = app
        if self.formatter is None:
            self.module = import_module("tygenie.alerts_list.formatter")
            self.formatter = "DefaultFormatter"
        else:
            self.module = import_module(
                f"tygenie.plugins.{self.formatter}.alerts_list_formatter"
            )
        self.formatter_fields = getattr(self.module, self.formatter).displayed_fields
        self.displayed_fields = self._get_final_displayed_fields()

    def _get_final_displayed_fields(self):
        displayed_fields = config.ty_config.tygenie.get("displayed_fields", {})
        fields_methods = displayed_fields.keys()
        if not fields_methods:
            displayed_fields = self.formatter_fields
            fields_methods = self.formatter_fields.keys()

        unknown_fields = []
        # Check if all custom fields have a corresponding method
        # 1/ check it exists in all fields
        # 2/ if it does not exists check the corresponding method name
        for method in fields_methods:
            if method in self.formatter_fields.keys():
                continue
            if self.formatter and str(method).lower() in dir(
                getattr(self.module, self.formatter)
            ):
                continue
            unknown_fields.append(method)

        if unknown_fields:
            ty_logger.logger.log(
                f"You defined some unknown fields that could not be displayed, they will be ignored: {unknown_fields}"
            )

        final_fields = [f for f in fields_methods if f not in unknown_fields]
        final_displayed_fields = {
            k: displayed_fields[k] for k in final_fields if k in displayed_fields
        }

        return final_displayed_fields

    def format(self) -> dict:
        to_format = {}
        formatter_fields = {}
        if self.formatter is not None:
            for f in self.formatter_fields.keys():
                if not hasattr(self.alert, f):
                    if f in self.displayed_fields:
                        to_format[f] = ""
                    formatter_fields[f] = ""
                    continue
                if f in self.displayed_fields:
                    to_format[f] = getattr(self.alert, f)
                formatter_fields[f] = getattr(self.alert, f)
            return getattr(self.module, self.formatter)(
                to_format=to_format,
                formatter_fields=formatter_fields,
                alert=self.alert,
                app=self.app,
            ).format()
        else:
            return {}


class DefaultFormatter(BaseFormatter):
    """Default alert list formatter"""



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.create_incident_payload_priority import CreateIncidentPayloadPriority
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_incident_payload_details import CreateIncidentPayloadDetails
    from ..models.create_incident_payload_status_page_entry import CreateIncidentPayloadStatusPageEntry
    from ..models.recipient import Recipient


T = TypeVar("T", bound="CreateIncidentPayload")


@_attrs_define
class CreateIncidentPayload:
    """
    Attributes:
        message (str): Message of the incident
        service_id (str): Service on which incident will be created.
        description (Union[Unset, str]): Description field of the incident that is generally used to provide a detailed
            information about the incident.
        responders (Union[Unset, List['Recipient']]): Responders that the incident will be routed to send notifications
        tags (Union[Unset, List[str]]): Tags of the incident.
        details (Union[Unset, CreateIncidentPayloadDetails]): Map of key-value pairs to use as custom properties of the
            incident
        priority (Union[Unset, CreateIncidentPayloadPriority]): Priority level of the incident
        note (Union[Unset, str]): Additional note that will be added while creating the incident
        status_page_entry (Union[Unset, CreateIncidentPayloadStatusPageEntry]): Status page entry fields. If this field
            is leaved blank, message and description of incident will be used for title and detail respectively.
        notify_stakeholders (Union[Unset, bool]): Indicate whether stakeholders are notified or not. Default value is
            false.
    """

    message: str
    service_id: str
    description: Union[Unset, str] = UNSET
    responders: Union[Unset, List["Recipient"]] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    details: Union[Unset, "CreateIncidentPayloadDetails"] = UNSET
    priority: Union[Unset, CreateIncidentPayloadPriority] = UNSET
    note: Union[Unset, str] = UNSET
    status_page_entry: Union[Unset, "CreateIncidentPayloadStatusPageEntry"] = UNSET
    notify_stakeholders: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        message = self.message

        service_id = self.service_id

        description = self.description

        responders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.responders, Unset):
            responders = []
            for responders_item_data in self.responders:
                responders_item = responders_item_data.to_dict()
                responders.append(responders_item)

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.details, Unset):
            details = self.details.to_dict()

        priority: Union[Unset, str] = UNSET
        if not isinstance(self.priority, Unset):
            priority = self.priority.value

        note = self.note

        status_page_entry: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.status_page_entry, Unset):
            status_page_entry = self.status_page_entry.to_dict()

        notify_stakeholders = self.notify_stakeholders

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "message": message,
                "serviceId": service_id,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if responders is not UNSET:
            field_dict["responders"] = responders
        if tags is not UNSET:
            field_dict["tags"] = tags
        if details is not UNSET:
            field_dict["details"] = details
        if priority is not UNSET:
            field_dict["priority"] = priority
        if note is not UNSET:
            field_dict["note"] = note
        if status_page_entry is not UNSET:
            field_dict["statusPageEntry"] = status_page_entry
        if notify_stakeholders is not UNSET:
            field_dict["notifyStakeholders"] = notify_stakeholders

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_incident_payload_details import CreateIncidentPayloadDetails
        from ..models.create_incident_payload_status_page_entry import CreateIncidentPayloadStatusPageEntry
        from ..models.recipient import Recipient

        d = src_dict.copy()
        message = d.pop("message")

        service_id = d.pop("serviceId")

        description = d.pop("description", UNSET)

        responders = []
        _responders = d.pop("responders", UNSET)
        for responders_item_data in _responders or []:
            responders_item = Recipient.from_dict(responders_item_data)

            responders.append(responders_item)

        tags = cast(List[str], d.pop("tags", UNSET))

        _details = d.pop("details", UNSET)
        details: Union[Unset, CreateIncidentPayloadDetails]
        if isinstance(_details, Unset):
            details = UNSET
        else:
            details = CreateIncidentPayloadDetails.from_dict(_details)

        _priority = d.pop("priority", UNSET)
        priority: Union[Unset, CreateIncidentPayloadPriority]
        if isinstance(_priority, Unset):
            priority = UNSET
        else:
            priority = CreateIncidentPayloadPriority(_priority)

        note = d.pop("note", UNSET)

        _status_page_entry = d.pop("statusPageEntry", UNSET)
        status_page_entry: Union[Unset, CreateIncidentPayloadStatusPageEntry]
        if isinstance(_status_page_entry, Unset):
            status_page_entry = UNSET
        else:
            status_page_entry = CreateIncidentPayloadStatusPageEntry.from_dict(_status_page_entry)

        notify_stakeholders = d.pop("notifyStakeholders", UNSET)

        create_incident_payload = cls(
            message=message,
            service_id=service_id,
            description=description,
            responders=responders,
            tags=tags,
            details=details,
            priority=priority,
            note=note,
            status_page_entry=status_page_entry,
            notify_stakeholders=notify_stakeholders,
        )

        create_incident_payload.additional_properties = d
        return create_incident_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.policy import Policy
from ...types import UNSET, Response, Unset


def _get_kwargs(
    policy_id: str,
    *,
    body: Policy,
    team_id: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    params: Dict[str, Any] = {}

    params["teamId"] = team_id

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "put",
        "url": f"/v2/policies/{policy_id}",
        "params": params,
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json;charset=UTF-8"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == 200:
        return None
    if response.status_code == 400:
        return None
    if response.status_code == 401:
        return None
    if response.status_code == 402:
        return None
    if response.status_code == 403:
        return None
    if response.status_code == 404:
        return None
    if response.status_code == 422:
        return None
    if response.status_code == 429:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    policy_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: Policy,
    team_id: Union[Unset, str] = UNSET,
) -> Response[Any]:
    """Update Policy

     Update alert policy with given id

    Args:
        policy_id (str):
        team_id (Union[Unset, str]):
        body (Policy):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        policy_id=policy_id,
        body=body,
        team_id=team_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    policy_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: Policy,
    team_id: Union[Unset, str] = UNSET,
) -> Response[Any]:
    """Update Policy

     Update alert policy with given id

    Args:
        policy_id (str):
        team_id (Union[Unset, str]):
        body (Policy):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        policy_id=policy_id,
        body=body,
        team_id=team_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.alert_policy_priority import AlertPolicyPriority
from ..models.policy_type import PolicyType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.alert_policy_details import AlertPolicyDetails
    from ..models.filter_ import Filter
    from ..models.responder import Responder
    from ..models.time_restriction_interval import TimeRestrictionInterval


T = TypeVar("T", bound="AlertPolicy")


@_attrs_define
class AlertPolicy:
    """
    Attributes:
        type (PolicyType): Type of the policy
        id (Union[Unset, str]):
        name (Union[Unset, str]): Name of the policy
        policy_description (Union[Unset, str]): Description of the policy
        team_id (Union[Unset, str]): TeamId of the policy
        filter_ (Union[Unset, Filter]): Defines the conditions that will be checked before applying rules and type of
            the operations that will be applied on conditions
        time_restrictions (Union[Unset, TimeRestrictionInterval]):
        enabled (Union[Unset, bool]): Activity status of the alert policy
        ignore_original_actions (Union[Unset, bool]):
        ignore_original_details (Union[Unset, bool]): If set to true, policy will ignore the original details of the
            alert. Default value is false
        ignore_original_responders (Union[Unset, bool]): If set to true, policy will ignore the original responders of
            the alert. Default value is false
        ignore_original_tags (Union[Unset, bool]): If set to true, policy will ignore the original tags of the alert.
            Default value is false
        ignore_original_teams (Union[Unset, bool]): If set to true, policy will ignore the original teams of the alert.
            Default value is false
        continue_ (Union[Unset, bool]): Will continue with other modify policies if set to true
        alias (Union[Unset, str]): Alias of the alert. You can use {{alias}} to refer to the original alias. Default
            value is {{alias}}
        description (Union[Unset, str]): Description of the alert. You can use {{description}} to refer to the original
            alert description. Default value is {{description}}
        entity (Union[Unset, str]): Entity field of the alert. You can use {{entity}} to refer to the original entity.
            Default value is {{entity}}
        message (Union[Unset, str]): Message of the alert
        source (Union[Unset, str]): Source field of the alert. You can use {{source}} to refer to the original source.
            Default value is {{source}}
        actions (Union[Unset, List[str]]): Alert actions as a list of strings to add to the alerts original actions
            value. If ignoreOriginalActions field is set to true, this will replace the original actions.
        responders (Union[Unset, List['Responder']]): Responders to add to the alerts original responders value as a
            list of teams or users. If ignoreOriginalResponders field is set to true, this will replace the original
            responders.
        tags (Union[Unset, List[str]]): Tags to add to the alerts original tags value as a list of strings. If
            ignoreOriginalTags field is set to true, this will replace the original tags.
        details (Union[Unset, AlertPolicyDetails]): Custom properties to add to the alerts original details value as a
            list of strings. If ignoreOriginalDetails field is set to true, this will replace the original details.
        priority (Union[Unset, AlertPolicyPriority]): Priority level of the alert
    """

    type: PolicyType
    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    policy_description: Union[Unset, str] = UNSET
    team_id: Union[Unset, str] = UNSET
    filter_: Union[Unset, "Filter"] = UNSET
    time_restrictions: Union[Unset, "TimeRestrictionInterval"] = UNSET
    enabled: Union[Unset, bool] = UNSET
    ignore_original_actions: Union[Unset, bool] = UNSET
    ignore_original_details: Union[Unset, bool] = UNSET
    ignore_original_responders: Union[Unset, bool] = UNSET
    ignore_original_tags: Union[Unset, bool] = UNSET
    ignore_original_teams: Union[Unset, bool] = UNSET
    continue_: Union[Unset, bool] = UNSET
    alias: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    entity: Union[Unset, str] = UNSET
    message: Union[Unset, str] = UNSET
    source: Union[Unset, str] = UNSET
    actions: Union[Unset, List[str]] = UNSET
    responders: Union[Unset, List["Responder"]] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    details: Union[Unset, "AlertPolicyDetails"] = UNSET
    priority: Union[Unset, AlertPolicyPriority] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        id = self.id

        name = self.name

        policy_description = self.policy_description

        team_id = self.team_id

        filter_: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.filter_, Unset):
            filter_ = self.filter_.to_dict()

        time_restrictions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.time_restrictions, Unset):
            time_restrictions = self.time_restrictions.to_dict()

        enabled = self.enabled

        ignore_original_actions = self.ignore_original_actions

        ignore_original_details = self.ignore_original_details

        ignore_original_responders = self.ignore_original_responders

        ignore_original_tags = self.ignore_original_tags

        ignore_original_teams = self.ignore_original_teams

        continue_ = self.continue_

        alias = self.alias

        description = self.description

        entity = self.entity

        message = self.message

        source = self.source

        actions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.actions, Unset):
            actions = self.actions

        responders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.responders, Unset):
            responders = []
            for responders_item_data in self.responders:
                responders_item = responders_item_data.to_dict()
                responders.append(responders_item)

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.details, Unset):
            details = self.details.to_dict()

        priority: Union[Unset, str] = UNSET
        if not isinstance(self.priority, Unset):
            priority = self.priority.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if policy_description is not UNSET:
            field_dict["policyDescription"] = policy_description
        if team_id is not UNSET:
            field_dict["teamId"] = team_id
        if filter_ is not UNSET:
            field_dict["filter"] = filter_
        if time_restrictions is not UNSET:
            field_dict["timeRestrictions"] = time_restrictions
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if ignore_original_actions is not UNSET:
            field_dict["ignoreOriginalActions"] = ignore_original_actions
        if ignore_original_details is not UNSET:
            field_dict["ignoreOriginalDetails"] = ignore_original_details
        if ignore_original_responders is not UNSET:
            field_dict["ignoreOriginalResponders"] = ignore_original_responders
        if ignore_original_tags is not UNSET:
            field_dict["ignoreOriginalTags"] = ignore_original_tags
        if ignore_original_teams is not UNSET:
            field_dict["ignoreOriginalTeams"] = ignore_original_teams
        if continue_ is not UNSET:
            field_dict["continue"] = continue_
        if alias is not UNSET:
            field_dict["alias"] = alias
        if description is not UNSET:
            field_dict["description"] = description
        if entity is not UNSET:
            field_dict["entity"] = entity
        if message is not UNSET:
            field_dict["message"] = message
        if source is not UNSET:
            field_dict["source"] = source
        if actions is not UNSET:
            field_dict["actions"] = actions
        if responders is not UNSET:
            field_dict["responders"] = responders
        if tags is not UNSET:
            field_dict["tags"] = tags
        if details is not UNSET:
            field_dict["details"] = details
        if priority is not UNSET:
            field_dict["priority"] = priority

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.alert_policy_details import AlertPolicyDetails
        from ..models.filter_ import Filter
        from ..models.responder import Responder
        from ..models.time_restriction_interval import TimeRestrictionInterval

        d = src_dict.copy()
        type = PolicyType(d.pop("type"))

        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        policy_description = d.pop("policyDescription", UNSET)

        team_id = d.pop("teamId", UNSET)

        _filter_ = d.pop("filter", UNSET)
        filter_: Union[Unset, Filter]
        if isinstance(_filter_, Unset):
            filter_ = UNSET
        else:
            filter_ = Filter.from_dict(_filter_)

        _time_restrictions = d.pop("timeRestrictions", UNSET)
        time_restrictions: Union[Unset, TimeRestrictionInterval]
        if isinstance(_time_restrictions, Unset):
            time_restrictions = UNSET
        else:
            time_restrictions = TimeRestrictionInterval.from_dict(_time_restrictions)

        enabled = d.pop("enabled", UNSET)

        ignore_original_actions = d.pop("ignoreOriginalActions", UNSET)

        ignore_original_details = d.pop("ignoreOriginalDetails", UNSET)

        ignore_original_responders = d.pop("ignoreOriginalResponders", UNSET)

        ignore_original_tags = d.pop("ignoreOriginalTags", UNSET)

        ignore_original_teams = d.pop("ignoreOriginalTeams", UNSET)

        continue_ = d.pop("continue", UNSET)

        alias = d.pop("alias", UNSET)

        description = d.pop("description", UNSET)

        entity = d.pop("entity", UNSET)

        message = d.pop("message", UNSET)

        source = d.pop("source", UNSET)

        actions = cast(List[str], d.pop("actions", UNSET))

        responders = []
        _responders = d.pop("responders", UNSET)
        for responders_item_data in _responders or []:
            responders_item = Responder.from_dict(responders_item_data)

            responders.append(responders_item)

        tags = cast(List[str], d.pop("tags", UNSET))

        _details = d.pop("details", UNSET)
        details: Union[Unset, AlertPolicyDetails]
        if isinstance(_details, Unset):
            details = UNSET
        else:
            details = AlertPolicyDetails.from_dict(_details)

        _priority = d.pop("priority", UNSET)
        priority: Union[Unset, AlertPolicyPriority]
        if isinstance(_priority, Unset):
            priority = UNSET
        else:
            priority = AlertPolicyPriority(_priority)

        alert_policy = cls(
            type=type,
            id=id,
            name=name,
            policy_description=policy_description,
            team_id=team_id,
            filter_=filter_,
            time_restrictions=time_restrictions,
            enabled=enabled,
            ignore_original_actions=ignore_original_actions,
            ignore_original_details=ignore_original_details,
            ignore_original_responders=ignore_original_responders,
            ignore_original_tags=ignore_original_tags,
            ignore_original_teams=ignore_original_teams,
            continue_=continue_,
            alias=alias,
            description=description,
            entity=entity,
            message=message,
            source=source,
            actions=actions,
            responders=responders,
            tags=tags,
            details=details,
            priority=priority,
        )

        alert_policy.additional_properties = d
        return alert_policy

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="EscalationRepeat")


@_attrs_define
class EscalationRepeat:
    """
    Attributes:
        wait_interval (Union[Unset, int]):
        count (Union[Unset, int]):
        reset_recipient_states (Union[Unset, bool]):
        close_alert_after_all (Union[Unset, bool]):
    """

    wait_interval: Union[Unset, int] = UNSET
    count: Union[Unset, int] = UNSET
    reset_recipient_states: Union[Unset, bool] = UNSET
    close_alert_after_all: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        wait_interval = self.wait_interval

        count = self.count

        reset_recipient_states = self.reset_recipient_states

        close_alert_after_all = self.close_alert_after_all

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if wait_interval is not UNSET:
            field_dict["waitInterval"] = wait_interval
        if count is not UNSET:
            field_dict["count"] = count
        if reset_recipient_states is not UNSET:
            field_dict["resetRecipientStates"] = reset_recipient_states
        if close_alert_after_all is not UNSET:
            field_dict["closeAlertAfterAll"] = close_alert_after_all

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        wait_interval = d.pop("waitInterval", UNSET)

        count = d.pop("count", UNSET)

        reset_recipient_states = d.pop("resetRecipientStates", UNSET)

        close_alert_after_all = d.pop("closeAlertAfterAll", UNSET)

        escalation_repeat = cls(
            wait_interval=wait_interval,
            count=count,
            reset_recipient_states=reset_recipient_states,
            close_alert_after_all=close_alert_after_all,
        )

        escalation_repeat.additional_properties = d
        return escalation_repeat

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ResponderType(str, Enum):
    ESCALATION = "escalation"
    SCHEDULE = "schedule"
    TEAM = "team"
    USER = "user"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.create_alert_payload_priority import CreateAlertPayloadPriority
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_alert_payload_details import CreateAlertPayloadDetails
    from ..models.recipient import Recipient


T = TypeVar("T", bound="CreateAlertPayload")


@_attrs_define
class CreateAlertPayload:
    """
    Attributes:
        message (str): Message of the alert
        user (Union[Unset, str]): Display name of the request owner
        note (Union[Unset, str]): Additional note that will be added while creating the alert
        source (Union[Unset, str]): Source field of the alert. Default value is IP address of the incoming request
        alias (Union[Unset, str]): Client-defined identifier of the alert, that is also the key element of alert
            deduplication.
        description (Union[Unset, str]): Description field of the alert that is generally used to provide a detailed
            information about the alert.
        responders (Union[Unset, List['Recipient']]): Responders that the alert will be routed to send notifications
        visible_to (Union[Unset, List['Recipient']]): Teams and users that the alert will become visible to without
            sending any notification
        actions (Union[Unset, List[str]]): Custom actions that will be available for the alert
        tags (Union[Unset, List[str]]): Tags of the alert
        details (Union[Unset, CreateAlertPayloadDetails]): Map of key-value pairs to use as custom properties of the
            alert
        entity (Union[Unset, str]): Entity field of the alert that is generally used to specify which domain alert is
            related to
        priority (Union[Unset, CreateAlertPayloadPriority]): Priority level of the alert
    """

    message: str
    user: Union[Unset, str] = UNSET
    note: Union[Unset, str] = UNSET
    source: Union[Unset, str] = UNSET
    alias: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    responders: Union[Unset, List["Recipient"]] = UNSET
    visible_to: Union[Unset, List["Recipient"]] = UNSET
    actions: Union[Unset, List[str]] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    details: Union[Unset, "CreateAlertPayloadDetails"] = UNSET
    entity: Union[Unset, str] = UNSET
    priority: Union[Unset, CreateAlertPayloadPriority] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        message = self.message

        user = self.user

        note = self.note

        source = self.source

        alias = self.alias

        description = self.description

        responders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.responders, Unset):
            responders = []
            for responders_item_data in self.responders:
                responders_item = responders_item_data.to_dict()
                responders.append(responders_item)

        visible_to: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.visible_to, Unset):
            visible_to = []
            for visible_to_item_data in self.visible_to:
                visible_to_item = visible_to_item_data.to_dict()
                visible_to.append(visible_to_item)

        actions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.actions, Unset):
            actions = self.actions

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.details, Unset):
            details = self.details.to_dict()

        entity = self.entity

        priority: Union[Unset, str] = UNSET
        if not isinstance(self.priority, Unset):
            priority = self.priority.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "message": message,
            }
        )
        if user is not UNSET:
            field_dict["user"] = user
        if note is not UNSET:
            field_dict["note"] = note
        if source is not UNSET:
            field_dict["source"] = source
        if alias is not UNSET:
            field_dict["alias"] = alias
        if description is not UNSET:
            field_dict["description"] = description
        if responders is not UNSET:
            field_dict["responders"] = responders
        if visible_to is not UNSET:
            field_dict["visibleTo"] = visible_to
        if actions is not UNSET:
            field_dict["actions"] = actions
        if tags is not UNSET:
            field_dict["tags"] = tags
        if details is not UNSET:
            field_dict["details"] = details
        if entity is not UNSET:
            field_dict["entity"] = entity
        if priority is not UNSET:
            field_dict["priority"] = priority

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_alert_payload_details import CreateAlertPayloadDetails
        from ..models.recipient import Recipient

        d = src_dict.copy()
        message = d.pop("message")

        user = d.pop("user", UNSET)

        note = d.pop("note", UNSET)

        source = d.pop("source", UNSET)

        alias = d.pop("alias", UNSET)

        description = d.pop("description", UNSET)

        responders = []
        _responders = d.pop("responders", UNSET)
        for responders_item_data in _responders or []:
            responders_item = Recipient.from_dict(responders_item_data)

            responders.append(responders_item)

        visible_to = []
        _visible_to = d.pop("visibleTo", UNSET)
        for visible_to_item_data in _visible_to or []:
            visible_to_item = Recipient.from_dict(visible_to_item_data)

            visible_to.append(visible_to_item)

        actions = cast(List[str], d.pop("actions", UNSET))

        tags = cast(List[str], d.pop("tags", UNSET))

        _details = d.pop("details", UNSET)
        details: Union[Unset, CreateAlertPayloadDetails]
        if isinstance(_details, Unset):
            details = UNSET
        else:
            details = CreateAlertPayloadDetails.from_dict(_details)

        entity = d.pop("entity", UNSET)

        _priority = d.pop("priority", UNSET)
        priority: Union[Unset, CreateAlertPayloadPriority]
        if isinstance(_priority, Unset):
            priority = UNSET
        else:
            priority = CreateAlertPayloadPriority(_priority)

        create_alert_payload = cls(
            message=message,
            user=user,
            note=note,
            source=source,
            alias=alias,
            description=description,
            responders=responders,
            visible_to=visible_to,
            actions=actions,
            tags=tags,
            details=details,
            entity=entity,
            priority=priority,
        )

        create_alert_payload.additional_properties = d
        return create_alert_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import json

from textual.app import ComposeResult
from textual.containers import HorizontalScroll
from textual.widget import Widget

from tygenie.widgets.buttons import Buttons


class AlertDetailsUrlButtonsBar(Widget):

    def compose(self) -> ComposeResult:
        self.display = False
        self.border_title = "Related links"
        with HorizontalScroll(id="alert_details_url_buttons_container"):
            yield Buttons(id="alert_details_url_buttons")

    def update(self, names_urls=[]):
        self.query_one("#alert_details_url_buttons", Buttons).update(names_urls)
        if names_urls:
            self.display = True

    def parse_content_and_update(self, content={}):
        def get_values(i):
            if isinstance(i, dict):
                return [
                    value for sub_item in i.values() for value in get_values(sub_item)
                ]
            elif isinstance(i, list):
                return [value for sub_item in i for value in get_values(sub_item)]
            else:
                return [i]

        content_dict = json.loads(content)
        return get_values(content_dict)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.callback_condition_field import CallbackConditionField
from ..models.callback_condition_operation import CallbackConditionOperation
from ..types import UNSET, Unset

T = TypeVar("T", bound="CallbackCondition")


@_attrs_define
class CallbackCondition:
    """
    Attributes:
        field (CallbackConditionField):
        operation (CallbackConditionOperation):
        expected_value (str):
        not_ (Union[Unset, bool]):
        order (Union[Unset, int]):
    """

    field: CallbackConditionField
    operation: CallbackConditionOperation
    expected_value: str
    not_: Union[Unset, bool] = UNSET
    order: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field = self.field.value

        operation = self.operation.value

        expected_value = self.expected_value

        not_ = self.not_

        order = self.order

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "field": field,
                "operation": operation,
                "expectedValue": expected_value,
            }
        )
        if not_ is not UNSET:
            field_dict["not"] = not_
        if order is not UNSET:
            field_dict["order"] = order

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        field = CallbackConditionField(d.pop("field"))

        operation = CallbackConditionOperation(d.pop("operation"))

        expected_value = d.pop("expectedValue")

        not_ = d.pop("not", UNSET)

        order = d.pop("order", UNSET)

        callback_condition = cls(
            field=field,
            operation=operation,
            expected_value=expected_value,
            not_=not_,
            order=order,
        )

        callback_condition.additional_properties = d
        return callback_condition

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.schedule_meta import ScheduleMeta
    from ..models.timeline import Timeline


T = TypeVar("T", bound="ScheduleTimeline")


@_attrs_define
class ScheduleTimeline:
    """
    Attributes:
        field_parent (Union[Unset, ScheduleMeta]):
        start_date (Union[Unset, datetime.datetime]):
        end_date (Union[Unset, datetime.datetime]):
        final_timeline (Union[Unset, Timeline]):
        base_timeline (Union[Unset, Timeline]):
        override_timeline (Union[Unset, Timeline]):
        forwarding_timeline (Union[Unset, Timeline]):
    """

    field_parent: Union[Unset, "ScheduleMeta"] = UNSET
    start_date: Union[Unset, datetime.datetime] = UNSET
    end_date: Union[Unset, datetime.datetime] = UNSET
    final_timeline: Union[Unset, "Timeline"] = UNSET
    base_timeline: Union[Unset, "Timeline"] = UNSET
    override_timeline: Union[Unset, "Timeline"] = UNSET
    forwarding_timeline: Union[Unset, "Timeline"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field_parent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.field_parent, Unset):
            field_parent = self.field_parent.to_dict()

        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        final_timeline: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.final_timeline, Unset):
            final_timeline = self.final_timeline.to_dict()

        base_timeline: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.base_timeline, Unset):
            base_timeline = self.base_timeline.to_dict()

        override_timeline: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.override_timeline, Unset):
            override_timeline = self.override_timeline.to_dict()

        forwarding_timeline: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.forwarding_timeline, Unset):
            forwarding_timeline = self.forwarding_timeline.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if field_parent is not UNSET:
            field_dict["_parent"] = field_parent
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if final_timeline is not UNSET:
            field_dict["finalTimeline"] = final_timeline
        if base_timeline is not UNSET:
            field_dict["baseTimeline"] = base_timeline
        if override_timeline is not UNSET:
            field_dict["overrideTimeline"] = override_timeline
        if forwarding_timeline is not UNSET:
            field_dict["forwardingTimeline"] = forwarding_timeline

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.schedule_meta import ScheduleMeta
        from ..models.timeline import Timeline

        d = src_dict.copy()
        _field_parent = d.pop("_parent", UNSET)
        field_parent: Union[Unset, ScheduleMeta]
        if isinstance(_field_parent, Unset):
            field_parent = UNSET
        else:
            field_parent = ScheduleMeta.from_dict(_field_parent)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.datetime]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date)

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.datetime]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date)

        _final_timeline = d.pop("finalTimeline", UNSET)
        final_timeline: Union[Unset, Timeline]
        if isinstance(_final_timeline, Unset):
            final_timeline = UNSET
        else:
            final_timeline = Timeline.from_dict(_final_timeline)

        _base_timeline = d.pop("baseTimeline", UNSET)
        base_timeline: Union[Unset, Timeline]
        if isinstance(_base_timeline, Unset):
            base_timeline = UNSET
        else:
            base_timeline = Timeline.from_dict(_base_timeline)

        _override_timeline = d.pop("overrideTimeline", UNSET)
        override_timeline: Union[Unset, Timeline]
        if isinstance(_override_timeline, Unset):
            override_timeline = UNSET
        else:
            override_timeline = Timeline.from_dict(_override_timeline)

        _forwarding_timeline = d.pop("forwardingTimeline", UNSET)
        forwarding_timeline: Union[Unset, Timeline]
        if isinstance(_forwarding_timeline, Unset):
            forwarding_timeline = UNSET
        else:
            forwarding_timeline = Timeline.from_dict(_forwarding_timeline)

        schedule_timeline = cls(
            field_parent=field_parent,
            start_date=start_date,
            end_date=end_date,
            final_timeline=final_timeline,
            base_timeline=base_timeline,
            override_timeline=override_timeline,
            forwarding_timeline=forwarding_timeline,
        )

        schedule_timeline.additional_properties = d
        return schedule_timeline

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class WeekdayTimeRestrictionStartDay(str, Enum):
    FRIDAY = "friday"
    MONDAY = "monday"
    SATURDAY = "saturday"
    SUNDAY = "sunday"
    THURSDAY = "thursday"
    TUESDAY = "tuesday"
    WEDNESDAY = "wednesday"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.filter_type import FilterType

T = TypeVar("T", bound="Filter")


@_attrs_define
class Filter:
    """Defines the conditions that will be checked before applying rules and type of the operations that will be applied on
    conditions

        Attributes:
            type (FilterType):
    """

    type: FilterType
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = FilterType(d.pop("type"))

        filter_ = cls(
            type=type,
        )

        filter_.additional_properties = d
        return filter_

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.action_mapping_action import ActionMappingAction
from ..models.action_mapping_mapped_action import ActionMappingMappedAction
from ..types import UNSET, Unset

T = TypeVar("T", bound="ActionMapping")


@_attrs_define
class ActionMapping:
    """
    Attributes:
        action (ActionMappingAction):
        extra_field (Union[Unset, str]):
        extra_field_for_mapped_action (Union[Unset, str]):
        mapped_action (Union[Unset, ActionMappingMappedAction]):
    """

    action: ActionMappingAction
    extra_field: Union[Unset, str] = UNSET
    extra_field_for_mapped_action: Union[Unset, str] = UNSET
    mapped_action: Union[Unset, ActionMappingMappedAction] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        action = self.action.value

        extra_field = self.extra_field

        extra_field_for_mapped_action = self.extra_field_for_mapped_action

        mapped_action: Union[Unset, str] = UNSET
        if not isinstance(self.mapped_action, Unset):
            mapped_action = self.mapped_action.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "action": action,
            }
        )
        if extra_field is not UNSET:
            field_dict["extraField"] = extra_field
        if extra_field_for_mapped_action is not UNSET:
            field_dict["extraFieldForMappedAction"] = extra_field_for_mapped_action
        if mapped_action is not UNSET:
            field_dict["mappedAction"] = mapped_action

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        action = ActionMappingAction(d.pop("action"))

        extra_field = d.pop("extraField", UNSET)

        extra_field_for_mapped_action = d.pop("extraFieldForMappedAction", UNSET)

        _mapped_action = d.pop("mappedAction", UNSET)
        mapped_action: Union[Unset, ActionMappingMappedAction]
        if isinstance(_mapped_action, Unset):
            mapped_action = UNSET
        else:
            mapped_action = ActionMappingMappedAction(_mapped_action)

        action_mapping = cls(
            action=action,
            extra_field=extra_field,
            extra_field_for_mapped_action=extra_field_for_mapped_action,
            mapped_action=mapped_action,
        )

        action_mapping.additional_properties = d
        return action_mapping

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
